//#define LedMod //incompatible with the gearbox.

int readings = 10; //min value is 1. Increasing the number of readings increase the precision but takes more time.

#include "SRTPedals.h"
#include <Arduino.h>
#include "src/ADS1X15.h"
#include "src/Joystick.h"

//#define SAMD21

#ifdef LedMod
#include "src/Adafruit_NeoPixel.h"
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

#define LedPin1 8
#define LedPin2 11
#define LedPin3 14
#define numberofLEDS 8
#define brightness 20 //Higher values will make the Leds brighter.
Adafruit_NeoPixel accelLed (numberofLEDS, LedPin1, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel brakeLed (numberofLEDS, LedPin2, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel clutchLed (numberofLEDS, LedPin3, NEO_GRB + NEO_KHZ800);

//Adafruit_NeoPixel pedalLed[3] ={
//(7, LedPin1, NEO_GRB + NEO_KHZ800),
//(7, LedPin2, NEO_GRB + NEO_KHZ800),
//(7, LedPin3, NEO_GRB + NEO_KHZ800)};

#endif

SRTPedals_::SRTPedals_(byte _nbrPedal,
                       long _minInput0, long _minInput1, long _minInput2, long _minInput3,
                       long _maxInput0, long _maxInput1, long _maxInput2, long _maxInput3,
                       bool _isADS,
                       byte _ADSType,
                       Joystick_ * _joystickPtr)

  : numberOfPedals(_nbrPedal),
    isADS(_isADS), ADSType(_ADSType),
    joystickPtr(_joystickPtr) {
  minInput[0] = _minInput0;
  minInput[1] = _minInput1;
  minInput[2] = _minInput2;
  minInput[3] = _minInput3;
  maxInput[0] = _maxInput0;
  maxInput[1] = _maxInput1;
  maxInput[2] = _maxInput2;
  maxInput[3] = _maxInput3;


  if (isADS) {
    readings=1;
    if (ADSType == 1) {
      ADS1115 ADS(0x48);
    }
    else {
      ADS1015 ADS(0x48);
    }
  }
}

void SRTPedals_::begin() {
  #ifdef SAMD21
  analogReadResolution(12);
  SerialUSB.begin(9600);
#else
  Serial.begin(9600);
#endif
#ifdef LedMod
accelLed.begin();
brakeLed.begin();
clutchLed.begin();
accelLed.setBrightness(brightness);
brakeLed.setBrightness(brightness);
clutchLed.setBrightness(brightness);
fadeInOut();
#endif

joystickPtr->setAcceleratorRange(minInput[0], maxInput[0]);
joystickPtr->setBrakeRange(minInput[1], maxInput[1]);
joystickPtr->setRzAxisRange(minInput[2], maxInput[2]);

  if (isADS) {
    if (ADSType == 1) {
      ADS1.begin();

      ADS1.setMode(1);
      ADS1.setDataRate(7); //6 :475/s, 7 :860/s
    }
    else {
      ADS0.begin();
      ADS0.setMode(1);
      ADS0.setDataRate(7); //6 :475/s, 7 :860/s
    }
  }
}

void SRTPedals_::setPedals(bool sendDebug) {
  for (int i = 0; i < numberOfPedals; i++) {
    setRawRead(i);
    if (rawReadPedal[i] != lastReadPedal[i]) {
      setAxis(i);
    }
    #ifdef LedMod
      if(i==0){
     //accelLed.clear();

      for (int j = 0; j < numberofLEDS; j++) {
          accelLed.setPixelColor(j,accelLed.Color(0, 0.25*LedMap(i,j),0));
      }
      accelLed.show();
    }
          if(i==1){
     //brakeLed.clear();
      for (int j = 0; j < numberofLEDS; j++) {

          brakeLed.setPixelColor(j,brakeLed.Color(0.25*LedMap(i,j), 0, 0));
      }
      brakeLed.show();
    }
          if(i==2){
     //clutchLed.clear();

      for (int j = 0; j < numberofLEDS; j++) {
          clutchLed.setPixelColor(j,brakeLed.Color(0.25*LedMap(i,j)*(i+1==2), 0.25*LedMap(i,j)*(i+1==1), 0.25*LedMap(i,j)*(i+1==3)));
      }
      clutchLed.show();
    }
#endif
    if (sendDebug) {

      if (rawReadPedal[i] < minRead[i]) {
        minRead[i] = rawReadPedal[i];
      }
      if (rawReadPedal[i] > maxRead[i]) {
        maxRead[i] = rawReadPedal[i];
      }
  int pourcentage = constrain(map(rawReadPedal[i],minInput[i],maxInput[i],0,100),0,100) ;

if(i==0){Serial.print("Throttle - ");}
if(i==1){Serial.print("Brake    - ");}
if(i==2){Serial.print("Clutch   - ");}
      #ifdef SAMD21
SerialUSB.println((String)pourcentage + "%. Raw value : " + rawReadPedal[i] + ". Min is : " + minRead[i] + ". Max is : " + maxRead[i] );
      #else

      Serial.println((String)pourcentage + "%. raw value : " + rawReadPedal[i] + ". Min is : " + minRead[i] + ". Max is : " + maxRead[i] );
      #endif
      
    }

  }
  if (sendDebug) {Serial.println("");}
}


byte SRTPedals_::getPedalNumber() {
  return numberOfPedals;
}

void SRTPedals_::setAxis(int i) {
  if (i == 0) {
    joystickPtr->setAccelerator(rawReadPedal[i]);
  }
  if (i == 1) {
    joystickPtr->setBrake(rawReadPedal[i]);
  }
  if (i == 2) {
    joystickPtr->setRzAxis(rawReadPedal[i]);
  }
  }
#ifdef LedMod
long SRTPedals_::LedMap(int pedal, int led) {
  
  long ledMapByte = constrain(rawReadPedal[pedal],minInput[pedal],maxInput[pedal]);

  ledMapByte = map(ledMapByte, minInput[pedal], maxInput[pedal], 0, 300);
  if(ledMapByte <led*300/numberofLEDS){
    ledMapByte=0;}

    else{
     ledMapByte = map(ledMapByte, 300*led/numberofLEDS, 300*(led+1)/numberofLEDS, 0, 300); 
      }
        if(ledMapByte >(led+1)*300/numberofLEDS)
  {ledMapByte = 300;}
    
  
  return ledMapByte;
}
#endif
void   SRTPedals_::setRawRead(int i) {
  if (isADS) {
    if (ADSType == 1) {
      rawReadPedal[i] = ADS1.readADC(i);
    }
    else {
      rawReadPedal[i] = ADS0.readADC(i);
    }
  }
  else {

    rawReadPedal[i] = analogRead(i);
    
    for (int j =0; j<readings-1;j++){

    rawReadPedal[i] += analogRead(i);
    
    }
  }
}
  #ifdef LedMod
void SRTPedals_::fadeInOut(){

  for (int h =0; h<200;h++){

  for (int i=0; i<3;i++){
        if(i==0){
     accelLed.clear();
      for (int j = 0; j < numberofLEDS; j++) {   accelLed.setPixelColor(j,accelLed.Color(0, myExp(h), 0)); }
      accelLed.show();
    }
          if(i==1){
     brakeLed.clear();
      for (int j = 0; j < numberofLEDS; j++) {          brakeLed.setPixelColor(j,brakeLed.Color(myExp(h), 0, 0));
      }
      brakeLed.show();
    }
          if(i==2){     clutchLed.clear();
      for (int j = 0; j < numberofLEDS; j++) {          clutchLed.setPixelColor(j,clutchLed.Color(0, 0, myExp(h)));
      }
      clutchLed.show();
    }
}
delay(5);

}

accelLed.clear();
brakeLed.clear();
clutchLed.clear();
}
#endif
int SRTPedals_::myExp(int i){
if (i<100){
return i;}
else{return 200-i;}

  
}
