#include "SRTmouse.h"
#include <Mouse.h>
#define TopSpeed  5
#include <Keyboard.h>


  bool invertX=  false;           //invert X axis. Default is false.
  bool invertY=  false;           //invert Y axis. Default is false.
  bool invertXY =  false ;          //invert X and Y axis. Default is false

#define threshold 35
#define MouseSensitivity 17 //set the speed

bool SRTMouse_::getMouseMode(){
  return mouseMode;
}

SRTMouse_::SRTMouse_(
  bool onlyShowValue,      //use this to calibrate the joystick. Disables the mouse behaviour.
  bool secure   
  ){
  securityMod = secure;
  XMousePin=A0;
  YMousePin=A1;
  SwitchMousePin=A2;
  startedToClick = 0;
  configMouseDelay = 500;
  }

void SRTMouse_::begin(){
  pinMode(XMousePin, INPUT);
  pinMode(YMousePin, INPUT);
  pinMode(SwitchMousePin, INPUT_PULLUP);

delay(3000); //security delay to flash the code
  Mouse.begin();
  Keyboard.begin();
}

bool SRTMouse_::setAction(long time){
  return true;
}

int SRTMouse_::readAxis(int thisAxis) {


  int reading = analogRead(thisAxis);

  if (abs(reading - SRTMouse_::analogcenter) < threshold) {
    reading = analogcenter;
  }

  reading = map(reading, 0, 1023, 0, MouseSensitivity);

  int distance = reading - MouseSensitivity / 2;
  if (distance == MouseSensitivity / 2) {
    distance = distance + TopSpeed;
  }
  if (distance == -MouseSensitivity / 2) {
    distance = distance - TopSpeed;
  }
  return distance;
}

void SRTMouse_::Arrow(unsigned long currentMillis){
  
  int Xreading = analogRead(XMousePin) - 512;
  int Yreading = analogRead(YMousePin) - 512;

  if (invertXY) {
    Xreading = analogRead(YMousePin) - 512;
    Yreading = analogRead(XMousePin) - 512;
  }


  if (invertY) {
    Yreading = map(Yreading, -512, 511, 511, -512);
  }

  if (abs(Xreading) < threshold) {
    Xreading = 0;
  }
  if (abs(Yreading) < threshold) {
    Yreading = 0;
  }

  Xreading = 0.5 * (Xreading - Yreading);
  Yreading = 0.5 * (Xreading + Yreading);

  if (currentMillis - keyPressedTime > keyPressedDelay) {

    Keyboard.releaseAll();
  }

  if (Xreading == Yreading) {
    return;
  }
  if (currentMillis - keyPressedTime > delayNextPress) {
    keyPressed = false;
  }

  if (!keyPressed && !onlyShowValue) {


    if (Xreading > 0) {
      if (Yreading > 0) {
        if (invertX) {
          Keyboard.press(KEY_LEFT_ARROW);
        }
        else {
          Keyboard.press(KEY_RIGHT_ARROW);
        }
        keyPressed = true;
        keyPressedTime = currentMillis;
        setAction(currentMillis);
      }

      else {
        if (!invertY) {
          Keyboard.press(KEY_UP_ARROW);
        }
        else {
          Keyboard.press(KEY_DOWN_ARROW);
        }
        keyPressed = true;
        keyPressedTime = currentMillis;
        setAction(currentMillis);
      }
    }
    else {
      if (Yreading > 0) {
        if (!invertY) {
          Keyboard.press(KEY_DOWN_ARROW);
        }
        else {
          Keyboard.press(KEY_UP_ARROW);
        }
        keyPressed = true;
        keyPressedTime = currentMillis;
        setAction(currentMillis);
      }

      else {
        if (invertX) {Keyboard.press(KEY_RIGHT_ARROW);}
        else { Keyboard.press(KEY_LEFT_ARROW);}
        keyPressed = true;
        keyPressedTime = currentMillis;
        SRTMouse_::setAction(currentMillis);
      }
    }
  }
}



bool SRTMouse_::isSecurityMod(){
  return securityMod;
}
void SRTMouse_::setMouse(bool sendData, unsigned long currentMillis){
  if (sendData) {
    if (SRTMouse_::getMouseMode()) {Serial.println((String) "Mouse Mode. Press the joystick for more than " + configMouseDelay + " ms to switch mode. ");}
    else {Serial.println((String)"Arrow Mode. Press the joystick for more than " + configMouseDelay + " ms to switch mode." );}
    Serial.println((String) "Mouse Position : " + analogRead(XMousePin) + ", " +analogRead(YMousePin)+"." );
    if (!digitalRead(SwitchMousePin)) {Serial.println((String)"." + startedToClick);}
  }

  if (!digitalRead(SwitchMousePin) && lastClickState == false) { //si on est enfoncé alors qu'avant on ne l'était pas, on note le temps de départ
    startedToClick = currentMillis;
    lastClickState = true;
  }

  if (digitalRead(SwitchMousePin) && lastClickState == true) { //si on détecte un relâchement
    if ( currentMillis - startedToClick > configMouseDelay) { // si on était enfoncé depuis longtemps :
      Serial.println((String) " delta is" + (currentMillis - startedToClick));
      mouseMode = !mouseMode;
      Keyboard.releaseAll();
    }
    else {if(!onlyShowValue){                                            // si on n'était pas enfoncé depuis longtemps, et pas en mode test :
      if (mouseMode) {
        Mouse.press(); 
      }
      else {
        Keyboard.press(KEY_RETURN);
      }}
      lastClick = currentMillis;
    }
    lastClickState = false;                                              //on note qu'on est plus enfoncé
  }
  if (currentMillis - lastClick > clickPressed) {
    if (mouseMode) {
      Mouse.release();
    }
    else {
      Keyboard.release(KEY_RETURN);
    }
  }

  
     if (!invertXY) {
    xReading = readAxis(XMousePin);
    yReading = readAxis(YMousePin);
  }
  else {
    xReading = readAxis(YMousePin);
    yReading = readAxis(XMousePin);
  }

  if (invertX) {
    xReading = map(xReading, -MouseSensitivity, MouseSensitivity, MouseSensitivity, -MouseSensitivity);
  }
  if (invertY) {
    yReading = map(yReading, -MouseSensitivity, MouseSensitivity, MouseSensitivity, -MouseSensitivity);
  }
  if (getMouseMode() && !onlyShowValue) {
    if (currentMillis - lastMove > 20) { //20 is the refreshRate
      Mouse.move(xReading, yReading, 0);
      lastMove = currentMillis;
    }
  }
  else {
    Arrow(currentMillis);
  }
   }
